local L = AceLibrary("AceLocale-2.2"):new("ClassLoot");

--[[ Slash command handlers ]]--
local opts = {
	type='group',
	args = {
		check = {
			type = 'text',
			name = L["Check an item (Local)"],
			desc = L["Display ClassLoot for an item locally"],
			usage = L["<item link>"],
			get = false,
			set = "ShowItemStars"
		},
		gcheck = {
			type = 'text',
			name = L["Check an item (Guild)"],
			desc = L["Display ClassLoot for an item in guild chat"],
			usage = L["<item link>"],
			get = false,
			set = "GuildCheckItem"
		},
		rcheck = {
			type = 'text',
			name = L["Check an item (Raid)"],
			desc = L["Display ClassLoot for an item in raid chat"],
			usage = L["<item link>"],
			get = false,
			set = "RaidCheckItem"
		}
	}
}

--[[ Set the module up ]]--
ClassLoot = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceHook-2.1");
ClassLoot:RegisterChatCommand(L["Slash-Commands"], opts);

ClassLoot:RegisterDB("ClassLootDB")
ClassLoot:RegisterDefaults("profile", {
	showTooltip = true,
	showBoss = true,
	showInstance = true
} );

--[[ Run when ClassLoot is initialized ]]--
function ClassLoot:OnInitialize()
	-- Add the Data Updated Date to the version for /cl about
	self.version = self.version.." "..ClassLoot_Data_Updated;
end

--[[ Run when ClassLoot is enabled ]]--
function ClassLoot:OnEnable()
	-- Hook tooltips if needed
	if (ClassLoot.db.profile.showTooltip) then
		self:HookScript(GameTooltip, "OnTooltipSetItem");
		self:HookScript(ItemRefTooltip, "OnTooltipSetItem");
	end
	-- Build Interface Options window
	self:CreateInterfaceOptions()
end

function ClassLoot:GuildCheckItem(item)
	if (IsInGuild()) then
		self:ShowItemStars(item, 'GUILD');
	else
		self:Print(L["Not in a guild!"]);
	end
end

function ClassLoot:RaidCheckItem(item)
	if (GetNumRaidMembers() >= 1) then
		self:ShowItemStars(item, 'RAID');
	else
		self:Print(L["Not in a raid!"]);
	end
end

-- Get Item Info
--	link - The item link
function ClassLoot:GetItemInfoFromLink(link)
	if (type(link) == "string") then
		return ClassLoot_Data[tonumber(link:match('item:(%d+)'))];
	end
end

-- Show Item Stars
--	itemName - The link for the item
--	target   - The chat target (LOCAL, RAID or GUILD)
function ClassLoot:ShowItemStars(itemName, target)

	-- Default to LOCAL if no target is set
	local target = target or 'LOCAL';
	
	local res = AceLibrary("LibPeriodicTable-3.1"):ItemSearch(itemName);
	local BB = LibStub("LibBabble-Boss-3.0"):GetLookupTable();
	local BZ = LibStub("LibBabble-Zone-3.0"):GetLookupTable();
	local instance, boss = nil;
	local output = {};
	
	-- Get the info table
	local itemInfo = self:GetItemInfoFromLink(itemName);
	if (itemInfo == nil) then
		-- Not in ClassLoot_Data
		self:Print(itemName..' '..L["could not be found"]);
		return
	end
	
	-- Header
	table.insert(output, L["ClassLoot info for"]..' '..itemName);
	
	-- Who drops it where
	if (res) then
		-- k = index, v = table name [InstanceLoot.InstanceName.BossName]
		for _,v in pairs(res) do
			local set, instance, boss = strsplit(".", v);
			if (set == "InstanceLoot") then		
				-- Three special cases where the "boss" is not the Boss!
				if (boss == "Dust Covered Chest") then boss = "Chess Event" end
				if (boss == "Cache of the Firelord") then boss = "Majordomo Executus" end
				if (boss == "Four Horsemen Chest") then boss = "The Four Horsemen" end


					-- Check for another special case, boss = "Trash Mobs"
					if (boss == "Trash Mobs") then
						table.insert(output, L["Dropped in"]..' '..BZ[instance]..' '..L["by"]..' '..L[boss]);
					else
						table.insert(output, L["Dropped in"]..' '..BZ[instance]..' '..L["by"]..' '..BB[boss]);
					end
				end
			end
		end
	end

	-- Build starTable, index = number of stars, value = string of classes
	local starTable = { [3] = nil; [4] = nil; [5] = nil; }
	for class, stars in pairs(itemInfo) do
		-- We are skipping all 0, 1 and 2 star items as theres no point showing them
		-- After all, they shouldn't have made it into the Data file anyway!
		if (stars > 2) then
			if (starTable[stars] == nil) then
				starTable[stars] = L[class];
			else
				starTable[stars] = starTable[stars]..', '..L[class];
			end
		end
	end
	
	-- Go backwards through the table from 5 stars down to 3 and build the output
	for i=5, 3, -1 do
		if (starTable[i] ~= nil) then
			table.insert(output, self:MakeStars(i)..starTable[i]);
		end
	end
	
	-- Ouptput the Item Info!
	for _,v in pairs(output) do
		if (target == 'LOCAL') then
			self:Print(v);
		else
			SendChatMessage(v, target);
		end
	end
end

-- Make Stars
--	count - Number of stars to return
function ClassLoot:MakeStars(count)
	local stars = '';
	for i=1, 6, 1 do
		if (i <= count) then
			stars = stars..'*';
		else
			stars = stars..'  ';
		end
	end
	return stars;
end

--[[ Tooltip Hook Handler to add ClassLoot info to items ]]--
function ClassLoot:OnTooltipSetItem(tooltip, ...)
	local item = select(2, tooltip:GetItem())	
	local itemInfo = ClassLoot:GetItemInfoFromLink(item);
	if (itemInfo and self.db.profile.showTooltip) then
		-- Build starTable, index = number of stars, value = string of classes
		local starTable = { [3] = nil; [4] = nil; [5] = nil; }
		for class, stars in pairs(itemInfo) do
			-- We are skipping all 0, 1 and 2 star items as theres no point showing them
			-- After all, they shouldn't have made it into the Data file anyway!
			if (stars > 2) then
				if (starTable[stars] == nil) then
					starTable[stars] = L[class];
				else
					starTable[stars] = starTable[stars]..', '..L[class];
				end
			end
		end

		tooltip:AddLine("|cffff0000ClassLoot:|r");
		-- Go backwards through the table from 5 stars down to 3 and build the output
		for i=5, 3, -1 do
			if (starTable[i] ~= nil) then
				tooltip:AddLine("|cff0099ff"..ClassLoot:MakeStars(i).."    "..starTable[i].."|r");
			end
		end
		
		if (ClassLoot.db.profile.showBoss or ClassLoot.db.profile.showInstance) then
			local res = AceLibrary("LibPeriodicTable-3.1"):ItemSearch(item);
			local BB = LibStub("LibBabble-Boss-3.0"):GetLookupTable();
			local BZ = LibStub("LibBabble-Zone-3.0"):GetLookupTable();
			local instances, bosses = nil
			
			-- Who drops it where
			if (res) then
				-- k = index, v = table name [InstanceLoot.InstanceName.BossName]
				for _, v in pairs(res) do
					local set, instance, boss = strsplit(".", v);
					if (set == "InstanceLoot") then
						-- Three special cases where the "boss" is not the Boss!
						if (boss == "Dust Covered Chest") then boss = "Chess Event" end
						if (boss == "Cache of the Firelord") then boss = "Majordomo Executus" end
						if (boss == "Four Horsemen Chest") then boss = "The Four Horsemen" end

						-- Localise 'boss' and 'instance'
						if (boss == "Trash Mobs") then
							boss = L[boss]
						else
							boss = BB[boss]
						end
						instance = BZ[instance]

						-- Concatenate to output strings
						if (not bosses) then
							bosses = boss
						else
							bosses = bosses..', '..boss
						end
						if (not instances) then
							instances = instance
						elseif (not instances:find(instance)) then
							instances = instances..', '..instance
						end	
					end
				end
			end
			-- Finally, append to the tooltip
			if (ClassLoot.db.profile.showBoss and bosses) then
				tooltip:AddLine("|cfff0e68c"..L["Boss"]..":        "..bosses.."|r");
			end
			if (ClassLoot.db.profile.showInstance and instances) then
				tooltip:AddLine("|cfff0e68c"..L["Instance"]..":  "..instances.."|r");
			end
		end
	end
	return self.hooks[tooltip].OnTooltipSetItem(tooltip, ...);
end

--[[ Interface Options Window ]]--
function ClassLoot:CreateInterfaceOptions()

	local cfgFrame = CreateFrame("FRAME", nil, UIParent);
	cfgFrame.name = L["ClassLoot"];
	
	local cfgFrameHeader = cfgFrame:CreateFontString("OVERLAY", nil, "GameFontNormalLarge");
	cfgFrameHeader:SetPoint("TOPLEFT", 15, -15);
	cfgFrameHeader:SetText(L["ClassLoot"]);

	local cfgShowTooltip = CreateFrame("CHECKBUTTON", "ClassLoot_cfgShowTooltip", cfgFrame, "InterfaceOptionsCheckButtonTemplate");
	ClassLoot_cfgShowTooltip:SetPoint("TOPLEFT", 20, -40);
	ClassLoot_cfgShowTooltipText:SetText(L["Enable ClassLoot Tooltips"]);
	ClassLoot_cfgShowTooltip:SetChecked(ClassLoot.db.profile.showTooltip);
	ClassLoot_cfgShowTooltip:SetScript("OnClick", function(self)
		ClassLoot.db.profile.showTooltip = not ClassLoot.db.profile.showTooltip
		-- Toggle the sub-options
		if (ClassLoot.db.profile.showTooltip) then
			-- Enable Sub-Options
			ClassLoot_cfgShowBossTooltip:Enable();
			ClassLoot_cfgShowInstanceTooltip:Enable();
			-- Hook the tooltips
			ClassLoot:HookScript(GameTooltip, "OnTooltipSetItem");
			ClassLoot:HookScript(ItemRefTooltip, "OnTooltipSetItem");
		else
			-- Disable Sub-Options
			ClassLoot_cfgShowBossTooltip:Disable();
			ClassLoot_cfgShowInstanceTooltip:Disable();
			-- Unhook the tooltips
			ClassLoot:Unhook(GameTooltip, "OnTooltipSetItem");
			ClassLoot:Unhook(ItemRefTooltip, "OnTooltipSetItem");
		end	
	end)
	
	local cfgShowBossTooltip = CreateFrame("CHECKBUTTON", "ClassLoot_cfgShowBossTooltip", cfgFrame, "InterfaceOptionsCheckButtonTemplate");
	ClassLoot_cfgShowBossTooltip:SetPoint("TOPLEFT", 40, -64);
	ClassLoot_cfgShowBossTooltipText:SetText(L["Display Boss Name"]);
	ClassLoot_cfgShowBossTooltip:SetChecked(ClassLoot.db.profile.showBoss);
	ClassLoot_cfgShowBossTooltip:SetScript("OnClick", function(self)
		ClassLoot.db.profile.showBoss = not ClassLoot.db.profile.showBoss
	end)	
	
	local cfgShowInstanceTooltip = CreateFrame("CHECKBUTTON", "ClassLoot_cfgShowInstanceTooltip", cfgFrame, "InterfaceOptionsCheckButtonTemplate");
	ClassLoot_cfgShowInstanceTooltip:SetPoint("TOPLEFT", 40, -88);
	ClassLoot_cfgShowInstanceTooltipText:SetText(L["Display Instance Name"]);
	ClassLoot_cfgShowInstanceTooltip:SetChecked(ClassLoot.db.profile.showInstance);
	ClassLoot_cfgShowInstanceTooltip:SetScript("OnClick", function(self)
		ClassLoot.db.profile.showInstance = not ClassLoot.db.profile.showInstance
	end)
	
	-- Check for disabled tooltips on startup
	if (not ClassLoot.db.profile.showTooltip) then
		ClassLoot_cfgShowBossTooltip:Disable();
		ClassLoot_cfgShowInstanceTooltip:Disable();
	end
	
	InterfaceOptions_AddCategory(cfgFrame);
end